package com.roshka.bootcamp004.poker;

import com.roshka.bootcamp004.mano.*;

public class Poker {
	public final String[] jugadas = {
			"Escalera Color",
			"Poker",
			"Full",
			"Color",
			"Escalera",
			"Trio",
			"Par doble",
			"Par",
			"Carta alta"
	};
	
	public Carta[] ordenarCartasPalo(Carta[] mano) {
		for(int i=0;i<mano.length-1;i++) {
			char menor = mano[i].obtenerCarta().charAt(1);
			int pos_menor = i;
			for(int j=i+1;j<mano.length;j++) {
				if(mano[j].obtenerCarta().charAt(1)>menor) {
					menor = mano[j].obtenerCarta().charAt(1);
					pos_menor = j;
				}
			}
			//Realizamos el intercambio
			if(pos_menor !=i) {
				Carta aux = mano[i];
				mano[i] = mano[pos_menor];
				mano[pos_menor] = aux;
			}
		}
		return mano;
	}
	
	public Carta[] ordenarCartasValor(Carta[] mano) {
		for(int i=0;i<mano.length-1;i++) {
			char menor = mano[i].obtenerCarta().charAt(0);
			int pos_menor = i;
			for(int j=i+1;j<mano.length;j++) {
				if(mano[j].compararValorCon(menor) == -1) {
					menor = mano[j].obtenerCarta().charAt(0);
					pos_menor = j;
				}
			}
			//Realizamos el intercambio
			if(pos_menor !=i) {
				Carta aux = mano[i];
				mano[i] = mano[pos_menor];
				mano[pos_menor] = aux;
			}
		}
		return mano;
	}
	
	protected boolean esEscaleraColor(Carta[] mano) {
		//Primero comprobamos que tengan todas las cartas el mismo palo
		if(esColor(mano)) {
			if(this.esEscalera(mano)) {
				return true;
			}
		}
		return false;
	}
	
	protected boolean esPoker(Carta[] mano) {
		char[] valores = new char[5];
		//Ordenamos las cartas por valor
		mano = this.ordenarCartasValor(mano);
		//Cargamos los valores de la mano
		for(int i = 0;i<mano.length;i++) {
			valores[i] = mano[i].obtenerCarta().charAt(0);
		}
		//Primer caso
		if(valores[0] == valores[1]) {
			if(valores[1]==valores[2]) {
				if(valores[2] == valores[3]) {
					return true;
				}
			}
		}
		//Segundo caso
		if(valores[1] == valores[2]) {
			if(valores[2]==valores[3]) {
				if(valores[3] == valores[4]) {
					return true;
				}
			}
		}
		return false;
	}
	
	protected boolean esFull(Carta[] mano) {
		char[] valores = new char[5];
		//Ordenamos las cartas por valor
		mano = this.ordenarCartasValor(mano);
		//Cargamos los valores de la mano
		for(int i = 0;i<mano.length;i++) {
			valores[i] = mano[i].obtenerCarta().charAt(0);
		}
		//Existen 2 casos
		//Caso 1
		if(valores[0] == valores[1]) {
			if(valores[1]==valores[2]) {
				if(valores[3]==valores[4]) {
					return true;
				}
			}
		}
		//Caso 2
		if(valores[2] == valores[3]) {
			if(valores[3]==valores[4]) {
				if(valores[0]==valores[1]) {
					return true;
				}
			}
		}
		return false;
	}
	
	protected boolean esColor(Carta[] mano){
		//Primero comprobamos que tengan todas las cartas el mismo palo
		char palo = mano[0].obtenerCarta().charAt(1);
		for(int i=1;i<5;i++) {
			if(mano[i].obtenerCarta().charAt(1) != palo) {
				return false;
			}
		}
		return true;
	}
	
	protected boolean esEscalera(Carta[] mano) {
		boolean resultado = true;
		char esR1 = 'p';
		char esR2 = 'p';
		//Ordenamos las cartas por valor
		mano = this.ordenarCartasValor(mano);
		//Obtenemos el valor de la primera carta y segunda carta
		//para saber si se obtiene escalera real
		esR1 = mano[0].obtenerCarta().charAt(0);
		esR2 = mano[1].obtenerCarta().charAt(0);
		//Existen dos casos, escalera real o no escalera real
		//Caso 1: Escalera real
		if(esR1 =='A' && esR2 == 'T') {
			for(int i = 1;i<mano.length-1 && resultado;i++) {
				if(!mano[i+1].esSuAnterior(mano[i])) {
					resultado = false;
				}
			}
		//Caso 2: No escalera real
		}else{
			for(int i = 0;i<mano.length-1 && resultado;i++) {
				if(!mano[i+1].esSuAnterior(mano[i])) {
					resultado = false;
				}
			}
		}
		return resultado;
	}
	
	protected boolean esTrio(Carta[] mano) {
		char[] valores = new char[5];
		//Ordenamos las cartas por valor
		mano = this.ordenarCartasValor(mano);
		//Cargamos los valores de la mano
		for(int i = 0;i<mano.length;i++) {
			valores[i] = mano[i].obtenerCarta().charAt(0);
		}
		//Existen 3 casos
		//Caso 1
		if(valores[0] == valores[1]) {
			if(valores[1]==valores[2]) {
				return true;
			}
		}
		//Caso 2
		if(valores[1] == valores[2]) {
			if(valores[2]==valores[3]) {
				return true;
			}
		}
		//Caso 3
		if(valores[2] == valores[3]) {
			if(valores[3]==valores[4]) {
				return true;
			}
		}
		return false;
	}
	
	protected boolean esParDoble(Carta[] mano) {
		char[] valores = new char[5];
		//Ordenamos las cartas por valor
		mano = this.ordenarCartasValor(mano);
		//Cargamos los valores de la mano
		for(int i = 0;i<mano.length;i++) {
			valores[i] = mano[i].obtenerCarta().charAt(0);
		}
		//Existen 2 casos
		//Caso 1
		if(valores[0] == valores[1]) {
			if(valores[2]==valores[3]) {
				return true;
			}else if(valores[3]==valores[4]) {
				return true;
			}
		}
		//Caso 2
		if(valores[1] == valores[2]) {
			if(valores[3]==valores[4]) {
				return true;
			}
		}
		return false;
	}
	
	protected boolean esPar(Carta[] mano) {
		char[] valores = new char[5];
		//Ordenamos las cartas por valor
		mano = this.ordenarCartasValor(mano);
		//Cargamos los valores de la mano
		for(int i = 0;i<mano.length;i++) {
			valores[i] = mano[i].obtenerCarta().charAt(0);
		}
		//Realizamos las comprobaciones
		for(int i = 0;i<mano.length-1;i++) {
			if(valores[i]==valores[i+1]) {
				return true;
			}
		}
		return false;
	}
	/**
	 * Entrega el tipo de mano que tiene el jugador
	 * @param mano El conjunto de objeto de tipo 'Carta'
	 * @return El tipo de mano
	 */
	public String mejorJugada(Carta[] mano)
	{
		//Ponemos como predeterminado que la mano es Carta Alta
		int resultado = 8;
		//Realizamos la comprobaciones
		if(this.esEscaleraColor(mano)) {
			resultado = 0;
		}else if(this.esPoker(mano)) {
			resultado = 1;
		}else if(this.esFull(mano)) {
			resultado = 2;
		}else if(this.esColor(mano)) {
			resultado = 3;
		}else if(this.esEscalera(mano)) {
			resultado = 4;
		}else if(this.esTrio(mano)) {
			resultado = 5;
		}else if(this.esParDoble(mano)) {
			resultado = 6;
		}else if(this.esPar(mano)){
			resultado = 7;
		}
		return jugadas[resultado];
	}

}
